package micro.main;

import java.awt.Color;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;

import javax.imageio.ImageIO;
import javax.swing.BoxLayout;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;

import hu.elte.txtuml.api.model.Action;
import hu.elte.txtuml.api.model.execution.ModelExecutor;
import micro.model.DoorClosed;
import micro.model.DoorOpened;
import micro.model.MicrowaveOven;
import micro.model.Start;
import micro.model.Stop;

public class GUI implements GUIInterface {
	static MicrowaveOven oven;
	static JFrame frame;
	static JPanel mainPanel;
	static JPanel lampPanel;
	static JPanel ovenPanel;
	static JPanel imagePanel;
	static JPanel buttonPanel;
	static JButton openButton;
	static JButton closeButton;
	static JButton startButton;
	static JButton stopButton;
	static JButton testButton;
	static BufferedImage lampOff;
	static BufferedImage lampOn;
	static JLabel lampOffLabel;
	static JLabel lampOnLabel;
	static BufferedImage magnetronOff;
	static BufferedImage magnetronOn;
	static JLabel magnetronOffLabel;
	static JLabel magnetronOnLabel;
	static BufferedImage ovenClosed;
	static BufferedImage ovenOpen;
	static JLabel ovenClosedLabel;
	static JLabel ovenOpenLabel;

	static void init() {
		oven = Action.create(MicrowaveOven.class);
		mainPanel = new JPanel();
		mainPanel.setLayout(new BoxLayout(mainPanel, BoxLayout.Y_AXIS));
		lampPanel = new JPanel();
		ovenPanel = new JPanel();
		imagePanel = new JPanel();
		buttonPanel = new JPanel();
		try {
			ovenClosed = ImageIO.read(new File("src/micro/images/ovenclosed.png"));
			ovenOpen = ImageIO.read(new File("src/micro/images/ovenopen.png"));
			lampOff = ImageIO.read(new File("src/micro/images/lampoff.jpg"));
			lampOn = ImageIO.read(new File("src/micro/images/lampon.jpg"));
			magnetronOff = ImageIO.read(new File("src/micro/images/magnetronoff.jpg"));
			magnetronOn = ImageIO.read(new File("src/micro/images/magnetronon.jpg"));
		} catch (IOException e) {
			e.printStackTrace();
		}
		ovenClosedLabel = new JLabel(new ImageIcon(ovenClosed));
		ovenOpenLabel = new JLabel(new ImageIcon(ovenOpen));
		lampOffLabel = new JLabel(new ImageIcon(lampOff));
		lampOnLabel = new JLabel(new ImageIcon(lampOn));
		magnetronOffLabel = new JLabel(new ImageIcon(magnetronOff));
		magnetronOnLabel = new JLabel(new ImageIcon(magnetronOn));
		ovenPanel.setBackground(new Color(255,255,255));
		ovenPanel.add(ovenClosedLabel);
		ovenPanel.add(ovenOpenLabel);
		ovenOpenLabel.setVisible(false);
		imagePanel.setBackground(new Color(255,255,255));
		imagePanel.add(lampOffLabel);
		imagePanel.add(lampOnLabel);
		lampOnLabel.setVisible(false);
		imagePanel.add(magnetronOffLabel);
		imagePanel.add(magnetronOnLabel);
		magnetronOnLabel.setVisible(false);
		openButton = new JButton("Open");
		closeButton = new JButton("Close");
		startButton = new JButton("Start");
		stopButton = new JButton("Stop");
		closeButton.setVisible(false);
		openButton.addActionListener(new ActionListener() {			
			@Override
			public void actionPerformed(ActionEvent arg0) {
				ovenClosedLabel.setVisible(false);
				ovenOpenLabel.setVisible(true);
				openButton.setVisible(false);
				closeButton.setVisible(true);
				Action.send(new DoorOpened(), oven);
			}
		});
		closeButton.addActionListener(new ActionListener() {			
			@Override
			public void actionPerformed(ActionEvent arg0) {
				ovenOpenLabel.setVisible(false);
				ovenClosedLabel.setVisible(true);
				closeButton.setVisible(false);
				openButton.setVisible(true);
				Action.send(new DoorClosed(), oven);
			}
		});
		startButton.addActionListener(new ActionListener() {			
			@Override
			public void actionPerformed(ActionEvent arg0) {
				Action.send(new Start(), oven);
			}
		});
		stopButton.addActionListener(new ActionListener() {			
			@Override
			public void actionPerformed(ActionEvent arg0) {
				Action.send(new Stop(), oven);
			}
		});
		buttonPanel.add(openButton);
		buttonPanel.add(closeButton);
		buttonPanel.add(startButton);
		buttonPanel.add(stopButton);
		mainPanel.add(ovenPanel);
		mainPanel.add(imagePanel);
		mainPanel.add(buttonPanel);
		frame = new JFrame();
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		frame.add(mainPanel);
		frame.pack();
		frame.setVisible(true);
		Action.start(oven);
	}

	@Override
	public void lampOn() {
		lampOffLabel.setVisible(false);
		lampOnLabel.setVisible(true);
	}

	@Override
	public void lampOff() {
		lampOnLabel.setVisible(false);
		lampOffLabel.setVisible(true);
	}

	@Override
	public void magnetronOn() {
		magnetronOffLabel.setVisible(false);
		magnetronOnLabel.setVisible(true);
	}

	@Override
	public void magnetronOff() {
		magnetronOnLabel.setVisible(false);
		magnetronOffLabel.setVisible(true);
	}

	public static void main(String[] args) {
		ModelExecutor.create().setTraceLogging(true).start(GUI::init);
	}
}

